@@ -49,6 +49,7 @@ AppUiBase::AppUiBase(lua_State *L0, int model)
 {
   L = L0;
   iModel = model;
+  isInkMode = false;
 
   iMouseIn = 0; // points
   iMouseFactor = 1.0;  // scale 1:1
@@ -156,9 +157,12 @@ void AppUiBase::buildMenus()
   addItem(EEditMenu, "Apply properties", "apply_properties");
   addItem(EEditMenu);
   addItem(EEditMenu, "Insert text box", "insert_text_box");
+  addItem(EEditMenu, "Change text width", "change_width");
   addItem(EEditMenu, "Edit object", "edit");
   addItem(EEditMenu, "Edit object as XML", "edit_as_xml");
-  addItem(EEditMenu, "Change text width", "change_width");
+  addItem(EEditMenu);
+  addItem(EEditMenu, "Edit group", "edit_group");
+  addItem(EEditMenu, "End group edit", "end_group_edit");
   addItem(EEditMenu);
   addItem(EEditMenu, "Document properties", "document_properties");
   addItem(EEditMenu, "Style sheets", "style_sheets");
@@ -180,8 +184,10 @@ void AppUiBase::buildMenus()
 
   addItem(EPropertiesMenu);
 
-  startSubMenu(EPropertiesMenu, "Text style", ESubmenuTextStyle);
+  startSubMenu(EPropertiesMenu, "Minipage style", ESubmenuTextStyle);
   iTextStyleMenu = endSubMenu();
+  startSubMenu(EPropertiesMenu, "Label style", ESubmenuLabelStyle);
+  iLabelStyleMenu = endSubMenu();
 
   startSubMenu(EPropertiesMenu, "Horizontal alignment");
   addSubItem("left", "horizontalalignment|left");
@@ -208,6 +214,8 @@ void AppUiBase::buildMenus()
   addItem(EModeMenu, "Rotate objects", "mode_rotate");
   addItem(EModeMenu, "Stretch objects (with Shift: scale objects)",
 	  "mode_stretch");
+  addItem(EModeMenu, "Shear objects", "mode_shear");
+  addItem(EModeMenu, "Move graph nodes", "mode_graph");
   addItem(EModeMenu, "Pan the canvas", "mode_pan");
   addItem(EModeMenu, "Shred objects", "mode_shredder");
   addItem(EModeMenu);
@@ -258,6 +266,7 @@ void AppUiBase::buildMenus()
   addItem(ESnapMenu, "Set origin && snap", "@set_origin_snap");
   addItem(ESnapMenu, "Hide axes", "@hide_axes");
   addItem(ESnapMenu, "Set direction", "@set_direction");
+  addItem(ESnapMenu, "Set tangent direction", "@set_tangent_direction");
   addItem(ESnapMenu, "Reset direction", "@reset_direction");
   addItem(ESnapMenu, "Set line", "@set_line");
   addItem(ESnapMenu, "Set line && snap", "@set_line_snap");
@@ -421,7 +430,7 @@ void AppUiBase::canvasObserverWheelMoved(double degrees, bool precise)
 
 void AppUiBase::canvasObserverToolChanged(bool hasTool)
 {
-  setActionsEnabled(!hasTool);
+  setActionsEnabled(!hasTool || isInkMode);
 }
 
 static void adjust(double &x, int mode, double factor)
@@ -522,6 +531,8 @@ void AppUiBase::luaAbsoluteButton(const char *s)
 
 void AppUiBase::luaAction(String name)
 {
+  if (isInkMode && iCanvas->tool() != nullptr)
+    return; // refuse any action while drawing ink
   if (name.left(12) == "coordinates|") {
     if (name.right(2) == "mm")
       iMouseIn = 1;
