@@ -83,7 +83,8 @@ public:
 
   // tags for submenus
   enum { ESubmenuGridSize = 1000, ESubmenuAngleSize,
-	 ESubmenuTextStyle, ESubmenuSelectLayer, ESubmenuMoveLayer,
+	 ESubmenuTextStyle, ESubmenuLabelStyle,
+	 ESubmenuSelectLayer, ESubmenuMoveLayer,
 	 ESubmenuFin };
 
 public:
@@ -97,6 +98,7 @@ public:
 
   void luaShowPathStylePopup(Vector v);
   void luaBookmarkSelected(int index);
+  inline void setInkMode(bool ink) { isInkMode = ink; }
 
 public:  // What platforms must implement:
   virtual WINID windowId() = 0;
@@ -169,6 +171,7 @@ protected:
   MENUHANDLE iSelectLayerMenu;
   MENUHANDLE iMoveToLayerMenu;
   MENUHANDLE iTextStyleMenu;
+  MENUHANDLE iLabelStyleMenu;
   MENUHANDLE iGridSizeMenu;
   MENUHANDLE iAngleSizeMenu;
 
@@ -185,6 +188,7 @@ protected:
   double iMouseFactor;
   int iUiScale;
   int iToolbarScale;
+  bool isInkMode;
 };
 
 // --------------------------------------------------------------------
