@@ -343,7 +343,8 @@ static NSImage *loadIcon(String action)
 	     @"zoom_in", @"zoom_out", @"fit_objects", @"fit_page",
 	     @"fit_width", @"keyboard",
 	     @"mode_select", @"mode_translate", @"mode_rotate",
-	     @"mode_stretch", @"mode_pan", @"mode_shredder",
+	     @"mode_stretch", @"mode_shear", @"mode_graph",
+	     @"mode_pan", @"mode_shredder",
 	     @"mode_label", @"mode_math", @"mode_paragraph",
 	     @"mode_marks", @"mode_rectangles1", @"mode_rectangles2",
 	     @"mode_rectangles3", @"mode_parallelogram", @"mode_lines",
@@ -359,7 +360,8 @@ static NSImage *loadIcon(String action)
 - (NSArray<NSString *> *) toolbarDefaultItemIdentifiers:(NSToolbar *) toolbar
 {
   return @[ @"mode_select", @"mode_translate", @"mode_rotate",
-	     @"mode_stretch", @"mode_pan", @"mode_shredder",
+	     @"mode_stretch", @"mode_shear", @"mode_graph",
+	     @"mode_pan", @"mode_shredder",
 	     @"mode_label", @"mode_math", @"mode_paragraph",
 	     @"mode_marks", @"mode_rectangles1", @"mode_rectangles2",
 	     @"mode_rectangles3", @"mode_parallelogram", @"mode_lines",
@@ -675,6 +677,9 @@ AppUi::AppUi(lua_State *L0, int model)
 	       forOrientation:NSLayoutConstraintOrientationHorizontal];
 
   [iWindow setContentView:iContent];
+
+  setCheckMark("coordinates|", "points");
+  setCheckMark("scaling|", "1");
 }
 
 AppUi::~AppUi()
