@@ -486,6 +486,8 @@ void AppUi::initUi()
   addTButton(hObjectTools, "mode_translate", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_rotate", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_stretch", BTNS_CHECKGROUP);
+  addTButton(hObjectTools, "mode_shear", BTNS_CHECKGROUP);
+  addTButton(hObjectTools, "mode_graph", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_pan", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_shredder", BTNS_CHECKGROUP);
   addTButton(hObjectTools, 0, BTNS_CHECKGROUP);
@@ -509,7 +511,7 @@ void AppUi::initUi()
   addTButton(hObjectTools, "mode_circle3", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_ink", BTNS_CHECKGROUP);
 
-#define MODE_BUTTONS 22
+#define MODE_BUTTONS 24
 
   // size is used for dropdown list!  Don't set to zero.
   for (int i = EUiGridSize; i <= EUiAngleSize; ++i)
