@@ -216,20 +216,14 @@ prefs.automatic_use_title = false
 -- How to start browser to show Ipe manual:
 -- (not used with Windows or Cocoa toolkit)
 
--- On Linux, there are various options depending on your Desktop:
+prefs.browser = "xdg-open %s"
+-- On Linux, there are some other options you can try if xdg-open
+-- doesn't work on your desktop:
 -- Gnome 2: gnome-open
--- prefs.browser = "gnome-open %s"
 -- Gnome 3: gvfs-open
-prefs.browser = "gvfs-open %s"
 -- KDE: kde-open
--- prefs.browser = "kde-open %s"
 -- XFCE: exo-open
--- prefs.browser = "exo-open %s"
--- "sensible-browser" starts a new browser
-  local engines = { "default", "pdftex", "xetex" } -- "luatex" }
-  local engines = { "default", "pdftex", "xetex" } -- "luatex" }
--- (which is not nice, ideally we'd like a tab in an open browser window)
--- prefs.browser = "sensible-browser %s &"
+-- Ubuntu/Debian: sensible-browser
 
 -- How to start onscreen keyboard
 if config.platform == "win" then
@@ -241,10 +235,67 @@ else
   prefs.keyboard = nil
 end
 
--- Do not highlight current selection in "ink" drawing mode?
-prefs.no_ink_highlight = false
 -- Disable all snap modes in "ink" drawing mode?
-prefs.no_ink_snap = false
+prefs.no_ink_snap = true
+
+----------------------------------------------------------------------
+--
+-- The following settings can be used to tune the use of the pen in
+-- ink mode.  It seems digitizers differ a lot, here you can adapt Ipe
+-- to your's.
+--
+-- If your digitizer generates lots of pen move events, set
+-- ink_min_movement to avoid making thousands of tiny ink segments
+-- (start with a value like 0.2).
+--
+-- If there is a lot of noise in pen recognition, try setting
+-- ink_smoothing to a positive value k (smoothing is then performed
+-- over 2k+1 samples).  Try 3, for instance.
+--
+-- After a complete ink stroke has been drawn, Ipe simplifies it to
+-- remove excess vertices, by computing a path with the minimal number
+-- of vertices that differs by at most ink_tolerance.
+--
+-- Ink strokes that are very short (like a dot on an i) tend to appear
+-- too small. Use ink_dot_length and ink_dot_wider to enlarge them.
+--
+-- Finally, you can set ink_spline to have your ink strokes converted
+-- to a spline (rather than a polygonal curve).
+--
+----------------------------------------------------------------------
+
+-- Minimal distance for movement to update ink stroke
+prefs.ink_min_movement = 0
+
+-- To smooth ink strokes, perform sliding average over how many samples?
+prefs.ink_smoothing = 0
+
+-- When ink stroke is simplified, what is the tolerance?
+prefs.ink_tolerance = 0.3
+
+-- Threshold for considering an ink stroke a tap with little movement:
+prefs.ink_dot_length = 10.0
+
+-- When user taps the screen in ink mode with no or little movement,
+-- how much should the pen width be enlarged?
+-- nil means use current pen width
+prefs.ink_dot_wider = 2.0
+
+-- Turn ink strokes into splines?
+prefs.ink_spline = false
+
+----------------------------------------------------------------------
+
+-- Auto-exporting when document is being saved
+-- If this is set, then every time you save in xml format, say to
+-- mydoc.ipe, the same document is EXPORTED to mydoc.pdf.
+-- Careful: if mydoc.pdf exists, it is silently overwritten.
+-- mydoc.pdf cannot be opened by Ipe (it is an exported copy.
+
+prefs.auto_export_to_pdf = false
+
+----------------------------------------------------------------------
+
 -- Extended properties menu, perhaps useful for tablets:
 prefs.tablet_menu = false
 
