@@ -78,7 +78,10 @@ shortcuts = {
   mode_math = "Shift+4",
   mode_paragraph = "G",
   mode_marks = "M",
-  mode_rectangles = "B",
+  mode_rectangles1 = "B",
+  mode_rectangles2 = nil,
+  mode_rectangles3 = nil,
+  mode_parallelogram = nil,
   mode_lines = "P",
   mode_polygons = "Shift+P",
   mode_arc1 = "A",
@@ -100,6 +103,7 @@ shortcuts = {
   set_origin_snap = "F1",
   hide_axes = "Ctrl+F1",
   set_direction = "F2",
+  set_tangent_direction = "Ctrl+F3",
   reset_direction = "Ctrl+F2",
   set_line = nil,
   set_line_snap = "F3",
