@@ -649,6 +649,121 @@ end
 
 ----------------------------------------------------------------------
 
+local function path_len(points)
+  local t = 0
+  for i=2,#points do
+    t = t + (points[i] - points[i-1]):len()
+  end
+  return t
+end
+
+-- perform moving average
+local function smooth_path(v)
+  local M = prefs.ink_smoothing
+  local w = { v[1] }
+  for i = 2,#v-1 do
+    local p = V(0,0)
+    for j = i-M, i+M do
+      local q
+      if j < 1 then q = v[1] elseif j > #v then q = v[#v] else q = v[i] end
+      p = p + q
+    end
+    w[#w+1] = (1/(2*M+1)) * p
+  end
+  w[#w+1] = v[#v]
+  return w
+end
+
+local function simplify_path(points, tolerance)
+  local marked = {}
+  marked[1] = true
+  marked[#points] = true
+  local stack = { { 1, #points } }
+
+  -- mark points to keep
+  while #stack > 0 do
+    local pair = table.remove(stack)
+    local first = pair[1]
+    local last = pair[2]
+    local max_dist = 0
+    local index = nil
+    local seg = ipe.Segment(points[first], points[last])
+    for i = first + 1,last do
+      local d = seg:distance(points[i])
+      if d > max_dist then
+        index = i
+        max_dist = d
+      end
+    end
+    if max_dist > tolerance then
+      marked[index] = true
+      stack[#stack + 1] = { first, index }
+      stack[#stack + 1] = { index, last }
+    end
+  end
+  -- only use marked vertices
+  local out = {}
+  for i, point in ipairs(points) do
+    if marked[i] then
+      out[#out+1] = point
+    end
+  end
+  return out
+end
+
+-- vector for control points around p2
+local function tang(p1, p2, p3)
+  return (p2-p1):normalized() + (p3 - p2):normalized()
+end
+
+-- first control point between p2 and p3
+local function cp1(p1, p2, p3)
+  local tangent = tang(p1, p2, p3):normalized()
+  local vlen = (p3 - p2):len() / 3.0
+  return p2 + vlen * tangent
+end
+
+-- second control point between p2 and p3
+local function cp2(p2, p3, p4)
+  local tangent = tang(p2, p3, p4):normalized()
+  local vlen = (p3 - p2):len() / 3.0
+  return p3 - vlen * tangent
+end
+
+local function compute_spline(w)
+  local shape = { }
+  if #w > 2 then
+    local cp = cp2(w[1], w[2], w[3])
+    local seg = { w[1], cp, w[2] }
+    seg.type = "spline"
+    shape[1] = seg
+  else
+    local seg = { w[1], w[2] }
+    seg.type = "segment"
+    shape[1] = seg
+  end
+  for i = 2,#w-2 do
+    local q1 = w[i-1]
+    local q2 = w[i]
+    local q3 = w[i+1]
+    local q4 = w[i+2]
+    local cpa = cp1(q1, q2, q3)
+    local cpb = cp2(q2, q3, q4)
+    local seg = { q2, cpa, cpb, q3}
+    seg.type = "spline"
+    shape[#shape + 1] = seg
+  end
+  if #w > 2 then
+    local cp = cp1(w[#w-2], w[#w-1], w[#w])
+    local seg = { w[#w-1], cp, w[#w] }
+    seg.type = "spline"
+    shape[#shape + 1] = seg
+  end
+  return shape
+end
+
+----------------------------------------------------------------------
+
 INKTOOL = {}
 INKTOOL.__index = INKTOOL
 
@@ -658,6 +773,7 @@ function INKTOOL:new(model)
   tool.model = model
   local v = model.ui:pos()
   tool.v = { v }
+  -- print("make ink tool")
   model.ui:shapeTool(tool)
   local s = model.doc:sheets():find("color", model.attributes.stroke)
   tool.setColor(s.r, s.g, s.b)
@@ -666,36 +782,65 @@ function INKTOOL:new(model)
   return tool
 end
 
-function INKTOOL:compute()
-  self.shape = { type="curve", closed=false }
-  for i = 2, #self.v do
-    self.shape[#self.shape + 1] = { type="segment", self.v[i-1], self.v[i] }
+function INKTOOL:compute(incremental)
+  local v = self.v
+  if incremental and #v > 2 then
+    self.shape[#self.shape+1]={ type="segment", v[#v-1], v[#v]}
+  else
+    self.shape = { type="curve", closed=false }
+    for i = 2, #v do
+      self.shape[#self.shape + 1] = { type="segment", v[i-1], v[i] }
+    end
   end
   self.setShape( { self.shape }, 0, self.w * self.model.ui:zoom())
 end
 
 function INKTOOL:mouseButton(button, modifiers, press)
-  self.model.ui:finishTool()
   if self.shape then
-     local obj = ipe.Path(self.model.attributes, { self.shape })
-     -- round linecaps are prettier for handwriting
-     obj:set("linecap", "round")
-     self.model:creation("create ink path", obj)
+    self.v = smooth_path(self.v)
+    self.v = simplify_path(self.v, prefs.ink_tolerance / self.model.ui:zoom())
   else
-     -- mouse was pressed and released without movement
-     local shape = { type="curve", closed=false,
-		     { type="segment", self.v[1], self.v[1] } }
-     local obj = ipe.Path(self.model.attributes, { shape })
-     -- must make round linecap, otherwise it will be invisible
-     obj:set("linecap", "round")
-     self.model:creation("create ink dot", obj)
+    self.v = { self.v[1], self.v[1] }
   end
+  if prefs.ink_spline then
+    self.shape = compute_spline(self.v)
+    self.shape.type = "curve"
+    self.shape.closed = false
+  else
+    self:compute(false)
+  end
+  local obj = ipe.Path(self.model.attributes, { self.shape })
+  -- round linecaps are prettier for handwriting
+  obj:set("linecap", "round")
+  obj:set("linejoin", "round")
+  if path_len(self.v) * self.model.ui:zoom() < prefs.ink_dot_length then
+    -- pen was pressed and released with no or very little movement
+    if prefs.ink_dot_wider then
+      obj:set("pen", self.w * prefs.ink_dot_wider)
+    end
+    self.model:creation("create ink dot", obj)
+  else
+    self.model:creation("create ink path", obj)
+  end
+  self.model:page():deselectAll()  -- ink isn't selected after creation
+  self.model.ui:finishTool()
 end
 
 function INKTOOL:mouseMove()
-  self.v[#self.v + 1] = self.model.ui:pos()
-  self:compute()
-  self.model.ui:update(false) -- update tool
+  local v1 = self.v[#self.v]
+  local v2 = self.model.ui:pos()
+  if (v1-v2):len() * self.model.ui:zoom() > prefs.ink_min_movement then
+    -- do not update if motion is too small
+    self.v[#self.v + 1] = v2
+    self:compute(true)
+    local r = ipe.Rect()
+    local offset = V(self.w, self.w)
+    r:add(v1)
+    r:add(v2)
+    r:add(r:bottomLeft() - offset)
+    r:add(r:topRight() + offset)
+    self.model.ui:update(r) -- update rectangle containing new segment
+  end
 end
 
 function INKTOOL:key(text, modifiers)
@@ -751,46 +896,23 @@ end
 
 ----------------------------------------------------------------------
 
-function MODEL:createText(mode)
-  local prompt
-  local pos = self.ui:pos()
+function MODEL:createText(mode, pos, width, pinned)
+  local prompt = "Enter Latex source"
+  local stylekind = "labelstyle"
+  local styleatt = self.attributes.labelstyle
+  local explainer = "create text"
   if mode == "math" then
     prompt = "Enter Latex source for math formula"
-  else
-    prompt = "Enter Latex source"
   end
-  local d = ipeui.Dialog(self.ui:win(), "Create text object")
-  d:add("label", "label", { label=prompt }, 1, 1, 1, 2)
-  d:add("text", "text", { syntax="latex", focus=true,
-			  spell_check=prefs.spell_check }, 0, 1, 1, 2)
-  addEditorField(d, "text")
-  d:addButton("ok", "&Ok", "accept")
-  d:addButton("cancel", "&Cancel", "reject")
-  d:setStretch("row", 2, 1)
-  d:setStretch("column", 2, 1)
-  d:set("ignore-escape", true)
-  if prefs.auto_external_editor then
-    externalEditor(d, "text")
+  if mode == "paragraph" then
+    styleatt = self.attributes.textstyle
+    stylekind = "textstyle"
+    explainer = "create text paragraph"
   end
-  if ((prefs.auto_external_editor and prefs.editor_closes_dialog)
-    or d:execute(prefs.editor_size)) then
-    local t = d:get("text")
-    if mode == "math" then
-      t = "$" .. t .. "$"
-    end
-    local obj = ipe.Text(self.attributes, t, pos)
-    self:creation("create text", obj)
-    self:autoRunLatex()
-  end
-end
-
-----------------------------------------------------------------------
-
-function MODEL:createParagraph(pos, width, pinned)
-  local styles = self.doc:sheets():allNames("textstyle")
+  local styles = self.doc:sheets():allNames(stylekind)
   local sizes = self.doc:sheets():allNames("textsize")
   local d = ipeui.Dialog(self.ui:win(), "Create text object")
-  d:add("label", "label", { label="Enter latex source" }, 1, 1, 1, 2)
+  d:add("label", "label", { label=prompt }, 1, 1, 1, 2)
   d:add("style", "combo", styles, -1, 3)
   d:add("size", "combo", sizes, -1, 4)
   d:add("text", "text", { syntax="latex", focus=true,
@@ -801,12 +923,16 @@ function MODEL:createParagraph(pos, width, pinned)
   d:setStretch("row", 2, 1)
   d:setStretch("column", 2, 1)
   d:set("ignore-escape", true)
-  local style = indexOf(self.attributes.textstyle, styles)
+  local style = indexOf(styleatt, styles)
   if not style then style = indexOf("normal", styles) end
   local size = indexOf(self.attributes.textsize, sizes)
   if not size then size = indexOf("normal", sizes) end
   d:set("style", style)
   d:set("size", size)
+  if mode == "math" then
+    d:set("style", "math")
+    d:setEnabled("style", false)
+  end
   if prefs.auto_external_editor then
     externalEditor(d, "text")
   end
@@ -817,15 +943,17 @@ function MODEL:createParagraph(pos, width, pinned)
     local size = sizes[d:get("size")]
     local obj = ipe.Text(self.attributes, t, pos, width)
     obj:set("textsize", size)
-    obj:set("textstyle", style)
+    obj:set(stylekind, style)
     if pinned then
       obj:set("pinned", "horizontal")
     end
-    self:creation("create text paragraph", obj)
+    self:creation(explainer, obj)
     self:autoRunLatex()
   end
 end
 
+----------------------------------------------------------------------
+
 function MODEL:action_insert_text_box()
   local layout = self.doc:sheets():find("layout")
   local p = self:page()
@@ -840,7 +968,7 @@ function MODEL:action_insert_text_box()
   if not r:isEmpty() and r:bottom() < layout.framesize.y then
     y = r:bottom() - layout.paragraph_skip
   end
-  self:createParagraph(ipe.Vector(0, y), layout.framesize.x, true)
+  self:createText("paragraph", ipe.Vector(0, y), layout.framesize.x, true)
 end
 
 ----------------------------------------------------------------------
@@ -870,7 +998,7 @@ function PARAGRAPHTOOL:mouseButton(button, modifiers, press)
   self.model.ui:finishTool()
   local pos = V(math.min(self.v[1].x, self.v[2].x), self.v[1].y)
   local wid = math.abs(self.v[2].x - self.v[1].x)
-  self.model:createParagraph(pos, wid)
+  self.model:createText("paragraph", pos, wid)
 end
 
 function PARAGRAPHTOOL:mouseMove()
@@ -890,6 +1018,151 @@ end
 
 ----------------------------------------------------------------------
 
+local function collect_edges(p, box)
+  local edges = {}
+  for i, obj, sel, layer in p:objects() do
+    if obj:type() == "path" then
+      local shape = obj:shape()
+      local m = obj:matrix()
+      if #shape == 1 and shape[1].type == "curve"
+	and shape[1].closed == false then
+	local curve = shape[1]
+	local head = curve[1][1]
+	local seg = curve[#curve]
+	local tail = seg[#seg]
+	if box:contains(head) then
+	  edges[#edges + 1] = { obj=obj, head=true, objno=i }
+	elseif box:contains(tail) then
+	  edges[#edges + 1] = { obj=obj, head=false, objno=i }
+	end
+      end
+    end
+  end
+  return edges
+end
+
+----------------------------------------------------------------------
+
+GRAPHTOOL = {}
+GRAPHTOOL.__index = GRAPHTOOL
+
+local function findClosestVertex(model)
+  local bound = prefs.close_distance
+  local pos = model.ui:unsnappedPos()
+  local p = model:page()
+
+  local closest
+  for i,obj,sel,layer in p:objects() do
+    if p:visible(model.vno, i) and not p:isLocked(layer) then
+      if obj:type() == "group" or obj:type() == "reference" or
+        obj:type() == "text" then
+	local d = p:distance(i, pos, bound)
+	if d < bound then closest = i; bound = d end
+      end
+    end
+  end
+
+  if closest then
+    -- deselect all, and select only closest object
+    p:deselectAll()
+    p:setSelect(closest, 1)
+    return true
+  else
+    return p:hasSelection()
+  end
+end
+
+function GRAPHTOOL:new(model)
+  if not findClosestVertex(model) then return end
+  local p = model:page()
+  self.prim = p:primarySelection()
+  local tool = {}
+  setmetatable(tool, GRAPHTOOL)
+  tool.model = model
+  tool.orig = model.ui:pos()
+  tool.box = p:bbox(self.prim)
+  tool.edges = collect_edges(p, tool.box)
+  model.ui:shapeTool(tool)
+  tool.setColor(1.0, 0.0, 0.0)
+  return tool
+end
+
+function GRAPHTOOL:compute()
+  self.t = self.model.ui:pos() - self.orig
+  self.shape = { boxshape(self.box:bottomLeft() + self.t,
+			  self.box:topRight() + self.t) }
+  for i = 1,#self.edges do
+    local obj = self.edges[i].obj
+    local shape = obj:shape()
+    transformShape(obj:matrix(), shape)
+    local curve = shape[1]
+    if self.edges[i].head then
+      curve[1][1] = curve[1][1] + self.t
+    else
+      local seg = curve[#curve]
+      seg[#seg] = seg[#seg] + self.t
+    end
+    self.shape[i+1] = curve
+  end
+end
+
+function GRAPHTOOL:mouseMove()
+  self:compute()
+  self.setShape(self.shape)
+  self.model.ui:update(false) -- update tool
+end
+
+local function apply_node_mode(t, doc)
+  local p = doc[t.pno]
+  p:transform(t.primary, ipe.Translation(t.translation))
+  for _,e in ipairs(t.edges) do
+    local obj = p[e.objno]
+    local shape = obj:shape()
+    transformShape(obj:matrix(), shape)
+    local curve = shape[1]
+    if e.head then
+      curve[1][1] = curve[1][1] + t.translation
+    else
+      local seg = curve[#curve]
+      seg[#seg] = seg[#seg] + t.translation
+    end
+    p[e.objno]:setShape(shape)
+  end
+end
+
+function GRAPHTOOL:mouseButton(button, modifiers, press)
+  if press then return end
+  self:compute()
+  self.model.ui:finishTool()
+  local edges = {}
+  for i = 1,#self.edges do
+    self.edges[i].obj = nil
+  end
+
+  local t = { label = "move graph vertex",
+	      pno = self.model.pno,
+	      vno = self.model.vno,
+	      primary = self.prim,
+	      original = self.model:page():clone(),
+	      translation = self.t,
+	      edges = self.edges,
+	      undo = revertOriginal,
+	      redo = apply_node_mode,
+	    }
+  self.model:register(t)
+end
+
+function GRAPHTOOL:key(text, modifiers)
+  if text == "\027" then
+    self.model.ui:finishTool()
+    return true
+  else
+    return false
+  end
+end
+
+----------------------------------------------------------------------
+
 CHANGEWIDTHTOOL = {}
 CHANGEWIDTHTOOL.__index = CHANGEWIDTHTOOL
 
@@ -966,12 +1239,14 @@ function MODEL:startModeTool(modifiers)
     self.ui:selectTool(self:page(), self.vno,
 		       prefs.select_distance, modifiers.shift)
   elseif (self.mode == "translate" or self.mode == "stretch"
-	  or self.mode == "rotate") then
+	  or self.mode == "rotate" or self.mode == "shear") then
     self:startTransform(self.mode, modifiers.shift)
   elseif self.mode == "pan" then
     self.ui:panTool(self:page(), self.vno)
   elseif self.mode == "shredder" then
     self:shredObject()
+  elseif self.mode == "graph" then
+    GRAPHTOOL:new(self)
   elseif self.mode:sub(1,10) == "rectangles" or self.mode == "parallelogram" then
     BOXTOOL:new(self, modifiers.shift, self.mode)
   elseif self.mode == "splinegons" then
@@ -988,7 +1263,7 @@ function MODEL:startModeTool(modifiers)
   elseif self.mode == "marks" then
     self:createMark()
   elseif self.mode == "label" or self.mode == "math" then
-    self:createText(self.mode)
+    self:createText(self.mode, self.ui:pos())
   elseif self.mode == "paragraph" then
     PARAGRAPHTOOL:new(self)
   else
@@ -1050,7 +1325,7 @@ function apply_text_edit(d, data, run_latex)
   local model = data.model
   local final = data.obj:clone()
   final:setText(d:get("text"))
-  if data.style then final:set("textstyle", data.styles[d:get("style")]) end
+  if data.style then final:set(data.stylekind, data.styles[d:get("style")]) end
   if data.size then final:set("textsize", data.sizes[d:get("size")]) end
   local t = { label="edit text",
 	      pno=model.pno,
@@ -1073,9 +1348,12 @@ end
 
 function MODEL:action_edit_text(prim, obj)
   local mp = obj:get("minipage")
+  local stylekind = "labelstyle"
+  if mp then stylekind = "textstyle" end
   local d = ipeui.Dialog(self.ui:win(), "Edit text object")
   local data = { model=self,
-		 styles = self.doc:sheets():allNames("textstyle"),
+		 stylekind = stylekind,
+		 styles = self.doc:sheets():allNames(stylekind),
 		 sizes = self.doc:sheets():allNames("textsize"),
 		 prim=prim,
 		 obj=obj,
@@ -1092,19 +1370,15 @@ function MODEL:action_edit_text(prim, obj)
   d:setStretch("column", 2, 1)
   d:set("text", obj:text())
   d:set("ignore-escape", true)
-  local style = nil
-  local size = nil
-  if mp then
-    data.style = indexOf(obj:get("textstyle"), data.styles)
-    if data.style then
-      d:add("style", "combo", data.styles, 1, 3)
-      d:set("style", data.style)
-    end
-    data.size = indexOf(obj:get("textsize"), data.sizes)
-    if data.size then
-      d:add("size", "combo", data.sizes, 1, 4)
-      d:set("size", data.size)
-    end
+  data.style = indexOf(obj:get(stylekind), data.styles)
+  if data.style then
+    d:add("style", "combo", data.styles, 1, 3)
+    d:set("style", data.style)
+  end
+  data.size = indexOf(obj:get("textsize"), data.sizes)
+  if data.size then
+    d:add("size", "combo", data.sizes, 1, 4)
+    d:set("size", data.size)
   end
   if prefs.auto_external_editor then
     externalEditor(d, "text")
@@ -1116,6 +1390,68 @@ function MODEL:action_edit_text(prim, obj)
   end
 end
 
+function MODEL:accept_group_text_edit(prim, group, t, tobj)
+  local els = group:elements()
+  els[t] = tobj
+  local final = ipe.Group(els)
+  -- copy properties
+  final:set("pinned", group:get("pinned"))
+  final:set("transformations", group:get("transformations"))
+  final:setMatrix(group:matrix())
+  final:setText(group:text())
+  final:setClip(group:clip())
+  final:set("decoration", group:get("decoration"))
+  local t = { label="edit text in group",
+	      pno=self.pno,
+	      vno=self.vno,
+	      primary=prim,
+	      original=group:clone(),
+	      final=final,
+	    }
+  t.undo = function (t, doc)
+	     doc[t.pno]:replace(t.primary, t.original)
+	   end
+  t.redo = function (t, doc)
+	     doc[t.pno]:replace(t.primary, t.final)
+	   end
+  self:register(t)
+  self:autoRunLatex()
+end
+
+function MODEL:action_edit_group_text(prim, obj)
+  local t = nil
+  for i = 1,obj:count() do
+    if obj:elementType(i) == "text" then t = i end
+  end
+  if not t then
+    self:warning("Cannot edit object",
+		 "Only groups containing text can be edited")
+    return
+  end
+  local tobj = obj:element(t)
+  local d = ipeui.Dialog(self.ui:win(), "Edit text in group object")
+  d:add("label", "label", { label="Edit latex source" }, 1, 1, 1, 2)
+  d:add("text", "text", { syntax="latex", focus=true,
+			  spell_check=prefs.spell_check}, 0, 1, 1, 4)
+  addEditorField(d, "text")
+  d:addButton("ok", "&Ok", "accept")
+  d:addButton("cancel", "&Cancel", "reject")
+  d:setStretch("row", 2, 1)
+  d:setStretch("column", 2, 1)
+  d:set("text", tobj:text())
+  d:set("ignore-escape", true)
+  if prefs.auto_external_editor then
+    externalEditor(d, "text")
+  end
+  if ((prefs.auto_external_editor and prefs.editor_closes_dialog)
+    or d:execute(prefs.editor_size)) then
+    if string.match(d:get("text"), "^%s*$") then return end
+    local final = tobj:clone()
+    final:setText(d:get("text"))
+    self:accept_group_text_edit(prim, obj, t, final)
+  end
+end
+
 function MODEL:action_edit()
   local p = self:page()
   local prim = p:primarySelection()
@@ -1125,14 +1461,132 @@ function MODEL:action_edit()
     self:action_edit_text(prim, obj)
   elseif obj:type() == "path" then
     self:action_edit_path(prim, obj)
+  elseif obj:type() == "group" then
+    self:action_edit_group_text(prim, obj)
   else
     self:warning("Cannot edit " .. obj:type() .. " object",
-		 "Only text objects and path objects can be edited")
+		 "Only text objects, path objects, and groups with text can be edited")
   end
 end
 
 ----------------------------------------------------------------------
 
+local function start_group_edit(t, doc)
+  local p = doc[t.pno]
+  local layers = p:layers()
+  local layerOfPrim = p:layerOf(t.primary)
+  local activeLayer = p:active(t.vno)
+  local activeIndex = indexOf(activeLayer, layers)
+  local g = p[t.primary]
+  local elements = g:elements()
+  local matrix = g:matrix()
+  local layer = "EDIT-GROUP"
+  if g:get("decoration") ~= "normal" then
+    layer = layer .. "<" .. g:get("decoration"):sub(12) .. ">"
+  end
+  if layerOfPrim ~= activeLayer then layer = layer .. "-" .. layerOfPrim end
+  while indexOf(layer, layers) do
+    layer = layer:sub(1,10) .. "*" .. layer:sub(11)
+  end
+  p:remove(t.primary)
+  p:addLayer(layer)
+  p:moveLayer(layer, activeIndex + 1)
+  for _,obj in ipairs(elements) do
+    p:insert(nil, obj, nil, layer)
+    p:transform(#p, matrix)
+  end
+  p:deselectAll()
+  p:setActive(t.vno, layer)
+  p:setVisible(t.vno, layer, true)
+  for _,l in ipairs(layers) do p:setLocked(l, true) end
+end
+
+local function end_group_edit(t, doc)
+  local p = doc[t.pno]
+  local layers = p:layers()
+  local activeLayer = p:active(t.vno)
+  local activeIndex = indexOf(activeLayer, layers)
+  local origActive
+  if activeIndex > 1 then
+    origActive = layers[activeIndex - 1]
+  else
+    origActive = layers[2]
+  end
+  local origGroup = nil
+  local deco, i = string.match(activeLayer, "^EDIT%-GROUP%**<([^>]+)>()")
+  if not deco then
+    deco = "normal"
+    local og = string.match(activeLayer, "^EDIT%-GROUP%**%-(.*)$")
+    if og then origGroup = og end
+  else
+    deco = "decoration/" .. deco
+    local og = string.match(activeLayer, "^EDIT%-GROUP%**<[^>]+>%-(.*)$")
+    if og then origGroup = og end
+  end
+  if not origGroup or not indexOf(origGroup, layers) then
+    origGroup = origActive
+  end
+  local elements = {}
+  for i, obj, sel, layer in p:objects() do
+    if layer == activeLayer then
+      elements[#elements + 1] = obj:clone()
+    end
+  end
+  for i = #p,1,-1 do
+    if p:layerOf(i) == activeLayer then
+      p:remove(i)
+    end
+  end
+  local group = ipe.Group(elements)
+  group:set("decoration", deco)
+  p:setLocked(origActive, false)
+  p:setLocked(origGroup, false)
+  p:setActive(t.vno, origActive)
+  p:removeLayer(activeLayer)
+  p:insert(nil, group, 1, origGroup)
+  if not string.match(origActive, "^EDIT") then
+    for _,l in ipairs(p:layers()) do p:setLocked(l, false) end
+  end
+end
+
+function MODEL:saction_edit_group()
+  local p = self:page()
+  local prim = p:primarySelection()
+  if p[prim]:type() ~= "group" then
+    self.ui:explain("primary selection is not a group")
+    return
+  end
+  local t = { label="start group edit",
+	      pno = self.pno,
+	      vno = self.vno,
+	      primary = prim,
+	      original = p:clone(),
+	      undo = revertOriginal,
+	      redo = start_group_edit,
+	    }
+  self:register(t)
+end
+
+function MODEL:action_end_group_edit()
+  local p = self:page()
+  if (not string.match(p:active(self.vno), "^EDIT%-GROUP")) or
+    p:countLayers() < 2 then
+    self:warning("Cannot end group edit",
+		 "Active layer is not a group edit layer")
+    return
+  end
+  local t = { label="end group edit",
+	      pno = self.pno,
+	      vno = self.vno,
+	      original = p:clone(),
+	      undo = revertOriginal,
+	      redo = end_group_edit,
+	    }
+  self:register(t)
+end
+
+----------------------------------------------------------------------
+
 PASTETOOL = {}
 PASTETOOL.__index = PASTETOOL
 
