@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -44,9 +44,81 @@
 
 using namespace ipe;
 
-typedef WINBOOL WINAPI (*LPFNGETGESTUREINFO)(HGESTUREINFO hGestureInfo,
-					     PGESTUREINFO pGestureInfo);
-static LPFNGETGESTUREINFO pGetGestureInfo = 0;
+// --------------------------------------------------------------------
+// some definitions from Windows 8
+
+#define WM_POINTERUPDATE 0x245 // 581
+#define WM_POINTERDOWN 0x246   // 582
+#define WM_POINTERUP 0x247     // 583
+
+typedef enum {
+  PT_POINTER   = 0x00000001,
+  PT_TOUCH     = 0x00000002,
+  PT_PEN       = 0x00000003,
+  PT_MOUSE     = 0x00000004,
+  PT_TOUCHPAD  = 0x00000005
+} POINTER_INPUT_TYPE;
+
+typedef struct {
+  POINTER_INPUT_TYPE         pointerType;
+  UINT32                     pointerId;
+  UINT32                     frameId;
+  // POINTER_FLAGS              pointerFlags;
+  UINT32                     pointerFlags;
+  HANDLE                     sourceDevice;
+  HWND                       hwndTarget;
+  POINT                      ptPixelLocation;
+  POINT                      ptHimetricLocation;
+  POINT                      ptPixelLocationRaw;
+  POINT                      ptHimetricLocationRaw;
+  DWORD                      dwTime;
+  UINT32                     historyCount;
+  INT32                      inputData;
+  DWORD                      dwKeyStates;
+  UINT64                     PerformanceCount;
+  // POINTER_BUTTON_CHANGE_TYPE ButtonChangeType;
+  UINT32                     ButtonChangeType;
+} POINTER_INFO;
+
+typedef struct {
+  POINTER_INFO pointerInfo;
+  UINT32       penFlags;
+  UINT32       penMask;
+  UINT32       pressure;
+  UINT32       rotation;
+  INT32        tiltX;
+  INT32        tiltY;
+} POINTER_PEN_INFO;
+
+// --------------------------------------------------------------------
+
+typedef WINBOOL WINAPI (*LPSetGestureConfig)(HWND hwnd, DWORD dwReserved,
+					     UINT cIDs,
+					     PGESTURECONFIG pGestureConfig,
+					     UINT cbSize);
+static LPSetGestureConfig pSetGestureConfig = nullptr;
+
+typedef WINBOOL WINAPI (*LPGetGestureInfo)(HGESTUREINFO hGestureInfo,
+					   PGESTUREINFO pGestureInfo);
+static LPGetGestureInfo pGetGestureInfo = nullptr;
+
+typedef WINBOOL WINAPI (*LPCloseGestureInfoHandle)(HGESTUREINFO hGestureInfo);
+static LPCloseGestureInfoHandle pCloseGestureInfoHandle = nullptr;
+
+typedef WINBOOL WINAPI (*LPGetPointerType)(UINT32 pointerId,
+					   POINTER_INPUT_TYPE *pointerType);
+static LPGetPointerType pGetPointerType = nullptr;
+
+typedef WINBOOL WINAPI (*LPGetPointerPenInfo)(UINT32 pointerId,
+					      POINTER_PEN_INFO *penInfo);
+static LPGetPointerPenInfo pGetPointerPenInfo = nullptr;
+
+typedef WINBOOL WINAPI (*LPGetPointerPenInfoHistory)(UINT32 pointerId,
+						     UINT32 *entriesCount,
+						     POINTER_PEN_INFO *penInfo);
+static LPGetPointerPenInfoHistory pGetPointerPenInfoHistory = nullptr;
+
+// --------------------------------------------------------------------
 
 const wchar_t Canvas::className[] = L"ipeCanvasWindowClass";
 
@@ -66,6 +138,10 @@ void Canvas::invalidate(int x, int y, int w, int h)
   r.bottom = y + h;
   // ipeDebug("invalidate %d %d %d %d\n", r.left, r.right, r.top, r.bottom);
   InvalidateRect(hwnd, &r, FALSE);
+  /*
+  if (immediate && !isTransient)
+    UpdateWindow(hwnd); // force immediate update
+  */
 }
 
 // --------------------------------------------------------------------
@@ -79,21 +155,20 @@ void Canvas::key(WPARAM wParam, LPARAM lParam)
     mod |= EControl;
   if (GetKeyState(VK_MENU) < 0)
     mod |= EAlt;
-  ipeDebug("Canvas::key(%x, %x) %x", wParam, lParam, mod);
+  // ipeDebug("Canvas::key(%x, %x) %x", wParam, lParam, mod);
   String t;
   t.append(char(wParam));
   iTool->key(t, mod);
 }
 
-void Canvas::button(int button, bool down, int mod, int xPos, int yPos)
+void Canvas::button(int button, bool down, int mod, Vector v)
 {
-  ipeDebug("Canvas::button %d %d %d %d", button, down, xPos, yPos);
   POINT p;
-  p.x = xPos;
-  p.y = yPos;
+  p.x = int(v.x);
+  p.y = int(v.y);
   ClientToScreen(hwnd, &p);
   iGlobalPos = Vector(p.x, p.y);
-  computeFifi(xPos, yPos);
+  computeFifi(v.x, v.y);
   mod |= iAdditionalModifiers;
   if (iTool)
     iTool->mouseButton(button | mod, down);
@@ -101,10 +176,10 @@ void Canvas::button(int button, bool down, int mod, int xPos, int yPos)
     iObserver->canvasObserverMouseAction(button | mod);
 }
 
-void Canvas::mouseMove(int xPos, int yPos)
+void Canvas::mouseMove(Vector v)
 {
   // ipeDebug("Canvas::mouseMove %d %d", xPos, yPos);
-  computeFifi(xPos, yPos);
+  computeFifi(v.x, v.y);
   if (iTool)
     iTool->mouseMove();
   if (iObserver)
@@ -128,8 +203,8 @@ void Canvas::updateSize()
 {
   RECT rc;
   GetClientRect(hwnd, &rc);
-  iWidth = rc.right;
-  iHeight = rc.bottom;
+  iBWidth = iWidth = rc.right;
+  iBHeight = iHeight = rc.bottom;
 }
 
 void Canvas::wndPaint()
@@ -146,6 +221,7 @@ void Canvas::wndPaint()
   int w = ps.rcPaint.right - x0;
   int h = ps.rcPaint.bottom - y0;
 
+  // ipeDebug("wndPaint: %d %d", w, h);
   HBITMAP bits = CreateCompatibleBitmap(hdc, w, h);
   HDC bitsDC = CreateCompatibleDC(hdc);
   SelectObject(bitsDC, bits);
@@ -189,10 +265,63 @@ void Canvas::mouseButton(Canvas *canvas, int button, bool down,
       mod |= EControl;
     if (GetKeyState(VK_MENU) < 0)
       mod |= EAlt;
-    canvas->button(button, down, mod, xPos, yPos);
+    canvas->button(button, down, mod, Vector(xPos, yPos));
   }
 }
 
+Vector Canvas::location(HWND hwnd, POINT q, POINT h)
+{
+  POINT p = q;
+  ScreenToClient(hwnd, &p);
+  return Vector(h.x / iHimetric.x + p.x - q.x,
+		h.y / iHimetric.y + p.y - q.y);
+}
+
+LRESULT Canvas::handlePointerEvent(HWND hwnd, UINT message,
+				   WPARAM wParam, LPARAM lParam)
+{
+  UINT32 pointerId = LOWORD(wParam);
+  POINTER_INPUT_TYPE pointerType;
+  if (pGetPointerType && pGetPointerType(pointerId, &pointerType)
+      && pointerType == PT_PEN) {
+    POINTER_PEN_INFO penInfo;
+    if (pGetPointerPenInfo(pointerId, &penInfo)) {
+      // bool incontact = penInfo.pointerInfo.pointerFlags & 0x4;
+      bool firstButton = penInfo.pointerInfo.pointerFlags & 0x10;
+      bool secondButton = penInfo.pointerInfo.pointerFlags & 0x20;
+      int bt = firstButton ? 1 : (secondButton ? 2: 0);
+      if (message == WM_POINTERDOWN)
+	iHimetric = Vector(double(penInfo.pointerInfo.ptHimetricLocationRaw.x)
+			   / penInfo.pointerInfo.ptPixelLocationRaw.x,
+			   double(penInfo.pointerInfo.ptHimetricLocationRaw.y)
+			   / penInfo.pointerInfo.ptPixelLocationRaw.y);
+      Vector v = location(hwnd, penInfo.pointerInfo.ptPixelLocationRaw,
+			  penInfo.pointerInfo.ptHimetricLocationRaw);
+      UINT32 entriesCount = penInfo.pointerInfo.historyCount;
+      if (message == WM_POINTERDOWN) {
+	button(bt, true, 0, v);
+      } else if (message == WM_POINTERUP) {
+	button(bt, false, 0, v);
+      } else {
+	if (entriesCount > 1) {
+	  POINTER_PEN_INFO *pi = new POINTER_PEN_INFO[entriesCount];
+	  if (pGetPointerPenInfoHistory(pointerId, &entriesCount, pi)) {
+	    for (int i = entriesCount - 1; i >= 0; --i) {
+	      isTransient = (i > 0);
+	      mouseMove(location(hwnd, pi[i].pointerInfo.ptPixelLocationRaw,
+				 pi[i].pointerInfo.ptHimetricLocationRaw));
+	    }
+	  }
+	  delete [] pi;
+	} else
+	  mouseMove(v);
+      }
+      return 0;
+    }
+  }
+  return DefWindowProc(hwnd, message, wParam, lParam);
+}
+
 LRESULT Canvas::handlePanGesture(DWORD flags, POINTS &p)
 {
   Vector v(p.x, -p.y);
@@ -223,7 +352,7 @@ LRESULT Canvas::handleZoomGesture(DWORD flags, POINTS &p, ULONG d)
     update();
     if (iObserver)
       // scroll wheel hasn't moved, but update display of ppi
-      iObserver->canvasObserverWheelMoved(0);
+      iObserver->canvasObserverWheelMoved(0, true);
   }
   return 0;
 }
@@ -255,7 +384,7 @@ LRESULT CALLBACK Canvas::wndProc(HWND hwnd, UINT message, WPARAM wParam,
   case WM_MOUSEWHEEL:
     if (canvas && canvas->iObserver)
       canvas->iObserver->canvasObserverWheelMoved
-	(GET_WHEEL_DELTA_WPARAM(wParam));
+	(GET_WHEEL_DELTA_WPARAM(wParam) / 120.0, true);
     return 0;
   case WM_PAINT:
     if (canvas)
@@ -266,29 +395,33 @@ LRESULT CALLBACK Canvas::wndProc(HWND hwnd, UINT message, WPARAM wParam,
       canvas->updateSize();
     break;
   case WM_GESTURE: {
-    if (!pGetGestureInfo)
+    if (!pGetGestureInfo || !canvas)
       break;  // GetGestureInfo not available (Windows Vista or older)
     GESTUREINFO gi = {0};
     gi.cbSize = sizeof(gi);
     BOOL res = pGetGestureInfo((HGESTUREINFO) lParam, &gi);
-    if (res && canvas) {
-      if (gi.dwID == GID_PAN)
-	return canvas->handlePanGesture(gi.dwFlags, gi.ptsLocation);
-      else if (gi.dwID == GID_ZOOM)
-	return canvas->handleZoomGesture(gi.dwFlags, gi.ptsLocation,
-					 gi.ullArguments);
-    }
+    if (!res)
+      break;
+    pCloseGestureInfoHandle((HGESTUREINFO) lParam);
+    if (gi.dwID == GID_PAN)
+      return canvas->handlePanGesture(gi.dwFlags, gi.ptsLocation);
+    if (gi.dwID == GID_ZOOM)
+      return canvas->handleZoomGesture(gi.dwFlags, gi.ptsLocation,
+				       gi.ullArguments);
     break; }
   case WM_MOUSEMOVE:
     if (canvas) {
       int xPos = GET_X_LPARAM(lParam);
       int yPos = GET_Y_LPARAM(lParam);
-      canvas->mouseMove(xPos, yPos);
+      canvas->mouseMove(Vector(xPos, yPos));
     }
     break;
   case WM_LBUTTONDOWN:
   case WM_LBUTTONUP:
-    mouseButton(canvas, 1, message == WM_LBUTTONDOWN, wParam, lParam);
+    // ignore touch events here
+    ipeDebug("extra = %x", GetMessageExtraInfo());
+    if ((GetMessageExtraInfo() & 0xffffff80) != 0xff515780)
+      mouseButton(canvas, 1, message == WM_LBUTTONDOWN, wParam, lParam);
     break;
   case WM_LBUTTONDBLCLK:
     mouseButton(canvas, 0x81, true, wParam, lParam);
@@ -311,10 +444,20 @@ LRESULT CALLBACK Canvas::wndProc(HWND hwnd, UINT message, WPARAM wParam,
     ipeDebug("Canvas MINMAX %ld %ld", mm->ptMinTrackSize.x,
 	     mm->ptMinTrackSize.y);
     break; }
+  case WM_POINTERDOWN:
+  case WM_POINTERUP:
+  case WM_POINTERUPDATE:
+    if (canvas && canvas->isInkMode)
+      return canvas->handlePointerEvent(hwnd, message, wParam, lParam);
+    break;
   case WM_DESTROY:
     ipeDebug("Windows canvas is destroyed");
     delete canvas;
     break;
+  // don't show in debug output:
+  case WM_SETCURSOR:
+  case WM_NCHITTEST:
+    break;
   default:
     // ipeDebug("Canvas wndProc(%d)", message);
     break;
@@ -338,6 +481,21 @@ Canvas::Canvas(HWND parent)
     exit(9);
   }
   assert(GetWindowLongPtr(hwnd, GWLP_USERDATA));
+  if (pSetGestureConfig) {
+    DWORD panWant = GC_PAN
+      | GC_PAN_WITH_SINGLE_FINGER_VERTICALLY
+      | GC_PAN_WITH_SINGLE_FINGER_HORIZONTALLY
+      | GC_PAN_WITH_GUTTER
+      | GC_PAN_WITH_INERTIA;
+    DWORD panBlock = 0;
+    GESTURECONFIG gestureConfig[] = {
+      { GID_PAN, panWant, panBlock },
+      { GID_ZOOM, GC_ZOOM, 0},
+      { GID_TWOFINGERTAP, 0, GC_TWOFINGERTAP }
+    };
+    pSetGestureConfig(hwnd, 0, 3, gestureConfig, sizeof(GESTURECONFIG));
+  }
+  isTransient = false;
 }
 
 void Canvas::init(HINSTANCE hInstance)
@@ -364,9 +522,20 @@ void Canvas::init(HINSTANCE hInstance)
 
   // Check if GetGestureInfo is available (i.e. at least Windows 7)
   HMODULE hDll = LoadLibraryA("user32.dll");
-  if (hDll)
-    pGetGestureInfo = (LPFNGETGESTUREINFO)
+  if (hDll) {
+    pSetGestureConfig = (LPSetGestureConfig)
+      GetProcAddress(hDll, "SetGestureConfig");
+    pGetGestureInfo = (LPGetGestureInfo)
       GetProcAddress(hDll, "GetGestureInfo");
+    pCloseGestureInfoHandle = (LPCloseGestureInfoHandle)
+      GetProcAddress(hDll, "CloseGestureInfoHandle");
+    pGetPointerType = (LPGetPointerType)
+      GetProcAddress(hDll, "GetPointerType");
+    pGetPointerPenInfo = (LPGetPointerPenInfo)
+      GetProcAddress(hDll, "GetPointerPenInfo");
+    pGetPointerPenInfoHistory = (LPGetPointerPenInfoHistory)
+      GetProcAddress(hDll, "GetPointerPenInfoHistory");
+  }
 }
 
 // --------------------------------------------------------------------
