@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2015  Otfried Cheong
+    Copyright (C) 1993-2016  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -69,31 +69,77 @@ using namespace ipe;
   into the string are byte indices, not Unicode character indices.
 */
 
-//! Construct an empty string.
-String::String()
+String::Imp *String::theEmptyString = { nullptr };
+
+String::Imp *String::emptyString() noexcept
 {
-  iImp = new Imp;
-  iImp->iRefCount = 1;
-  iImp->iSize = 0;
-  iImp->iCapacity = 32;
-  iImp->iData = new char[iImp->iCapacity];
+  if (theEmptyString == nullptr) {
+    theEmptyString = new Imp;
+    theEmptyString->iRefCount = 10; // always > 1
+    theEmptyString->iSize = 0;
+    theEmptyString->iCapacity = 0;
+    theEmptyString->iData = nullptr;
+  }
+  // increment every time it's requested to make sure it's never destroyed
+  ++theEmptyString->iRefCount;
+  return theEmptyString;
+}
+
+//! Construct an empty string.
+String::String() noexcept
+{
+  iImp = emptyString();
 }
 
 //! Construct a string by making copy of \a str.
-String::String(const char *str)
+String::String(const char *str) noexcept
 {
   int len = strlen(str);
-  iImp = new Imp;
-  iImp->iRefCount = 1;
-  iImp->iSize = len;
-  iImp->iCapacity = len + 32;
-  iImp->iData = new char[iImp->iCapacity];
-  memcpy(iImp->iData, str, iImp->iSize);
+  if (!len)
+    iImp = emptyString();
+  else {
+    iImp = new Imp;
+    iImp->iRefCount = 1;
+    iImp->iSize = len;
+    iImp->iCapacity = (len + 32) & ~15;
+    iImp->iData = new char[iImp->iCapacity];
+    memcpy(iImp->iData, str, iImp->iSize);
+  }
+}
+
+//! Construct string by taking ownership of given \a data.
+String String::withData(char *data, int len) noexcept
+{
+  if (!len)
+    len = strlen(data);
+  String r; // empty string
+  --r.iImp->iRefCount;
+  r.iImp = new Imp;
+  r.iImp->iRefCount = 1;
+  r.iImp->iSize = len;
+  r.iImp->iCapacity = len;
+  r.iImp->iData = data;
+  return r;
+}
+
+//! Construct string by making copy of \a str with given \a len.
+String::String(const char *str, int len) noexcept
+{
+  if (!len)
+    iImp = emptyString();
+  else {
+    iImp = new Imp;
+    iImp->iRefCount = 1;
+    iImp->iSize = len;
+    iImp->iCapacity = (len + 32) & ~15;
+    iImp->iData = new char[iImp->iCapacity];
+    memcpy(iImp->iData, str, iImp->iSize);
+  }
 }
 
 //! Copy constructor.
 //! This only copies the reference and takes constant time.
-String::String(const String &rhs)
+String::String(const String &rhs) noexcept
 {
   iImp = rhs.iImp;
   iImp->iRefCount++;
@@ -102,23 +148,26 @@ String::String(const String &rhs)
 //! Construct a substring.
 /*! \a index must be >= 0.  \a len can be negative or too large to
   return entire string. */
-String::String(const String &rhs, int index, int len)
+String::String(const String &rhs, int index, int len) noexcept
 {
   // actually available data
   int len1 = index < rhs.size() ? rhs.size() - index : 0;
   if (len < 0 || len1 < len)
     len = len1;
-  iImp = new Imp;
-  iImp->iRefCount = 1;
-  iImp->iSize = len;
-  iImp->iCapacity = len + 32;
-  iImp->iData = new char[iImp->iCapacity];
-  if (len > 0)
+  if (!len) {
+    iImp = emptyString();
+  } else {
+    iImp = new Imp;
+    iImp->iRefCount = 1;
+    iImp->iSize = len;
+    iImp->iCapacity = (len + 32) & ~15;
+    iImp->iData = new char[iImp->iCapacity];
     memcpy(iImp->iData, rhs.iImp->iData + index, len);
+  }
 }
 
 //! Assignment takes constant time.
-String &String::operator=(const String &rhs)
+String &String::operator=(const String &rhs) noexcept
 {
   if (iImp != rhs.iImp) {
     if (iImp->iRefCount == 1) {
@@ -133,7 +182,7 @@ String &String::operator=(const String &rhs)
 }
 
 //! Destruct string if reference count has reached zero.
-String::~String()
+String::~String() noexcept
 {
   if (iImp->iRefCount == 1) {
     delete [] iImp->iData;
@@ -143,10 +192,16 @@ String::~String()
 }
 
 //! Make a private copy of the string with \a n bytes to spare.
-/*! When a private copy has to be made an extra 32 bytes is ensured. */
-void String::detach(int n)
+/*! When a private copy has to be made an extra 32 bytes are ensured. */
+void String::detach(int n) noexcept
 {
-  if (iImp->iRefCount > 1 || (iImp->iSize + n > iImp->iCapacity)) {
+  if (iImp == theEmptyString) {
+    iImp = new Imp;
+    iImp->iRefCount = 1;
+    iImp->iSize = 0;
+    iImp->iCapacity = 16;
+    iImp->iData = new char[iImp->iCapacity];
+  } else if (iImp->iRefCount > 1 || (iImp->iSize + n > iImp->iCapacity)) {
     Imp *imp = new Imp;
     imp->iRefCount = 1;
     imp->iSize = iImp->iSize;
@@ -165,8 +220,10 @@ void String::detach(int n)
 }
 
 //! Return a C style string with final zero byte.
-const char *String::z() const
+const char *String::z() const noexcept
 {
+  if (iImp == theEmptyString)
+    return "";
   String *This = const_cast<String *>(this);
   if (iImp->iSize == iImp->iCapacity)
     This->detach(1);
@@ -176,7 +233,7 @@ const char *String::z() const
 
 //! Return index of first occurrence of ch.
 /*! Return -1 if character does not appear. */
-int String::find(char ch) const
+int String::find(char ch) const noexcept
 {
   for (int i = 0; i < size(); ++i)
     if (iImp->iData[i] == ch)
@@ -186,7 +243,7 @@ int String::find(char ch) const
 
 //! Return index of first occurrence of rhs.
 /*! Return -1 if not substring is not present. */
-int String::find(const char *rhs) const
+int String::find(const char *rhs) const noexcept
 {
   int s = strlen(rhs);
   for (int i = 0; i < size() - s; ++i)
@@ -195,9 +252,25 @@ int String::find(const char *rhs) const
   return -1;
 }
 
+//! Return line starting at position \a index.
+//! Index is updated to point to next line.
+String String::getLine(int &index) const noexcept
+{
+  int i = index;
+  while (i < size() && iImp->iData[i] != '\r' && iImp->iData[i] != '\n')
+    ++i;
+  String result = substr(index, i-index);
+  if (i < size() && iImp->iData[i] == '\r')
+    ++i;
+  if (i < size() && iImp->iData[i] == '\n')
+    ++i;
+  index = i;
+  return result;
+}
+
 //! Return index of last occurrence of ch.
 /*! Return -1 if character does not appear. */
-int String::rfind(char ch) const
+int String::rfind(char ch) const noexcept
 {
   for (int i = size() - 1; i >= 0; --i)
     if (iImp->iData[i] == ch)
@@ -206,14 +279,14 @@ int String::rfind(char ch) const
 }
 
 //! Make string empty.
-void String::erase()
+void String::erase() noexcept
 {
   detach(0);
   iImp->iSize = 0;
 }
 
 //! Append \a rhs to this string.
-void String::append(const String &rhs)
+void String::append(const String &rhs) noexcept
 {
   int n = rhs.size();
   detach(n);
@@ -221,8 +294,19 @@ void String::append(const String &rhs)
   iImp->iSize += n;
 }
 
+//! Append \a rhs to this string.
+void String::append(const char *rhs) noexcept
+{
+  int n = strlen(rhs);
+  if (n) {
+    detach(n);
+    memcpy(iImp->iData + iImp->iSize, rhs, n);
+    iImp->iSize += n;
+  }
+}
+
 //! Append \a ch to this string.
-void String::append(char ch)
+void String::append(char ch) noexcept
 {
   detach(1);
   iImp->iData[iImp->iSize++] = ch;
@@ -230,7 +314,7 @@ void String::append(char ch)
 
 //! Create substring at the right.
 /*! Returns the entire string if \a i is larger than its length. */
-String String::right(int i) const
+String String::right(int i) const noexcept
 {
   if (i < size())
     return String(*this, size() - i, i);
@@ -238,15 +322,29 @@ String String::right(int i) const
     return *this;
 }
 
+//! Does string start with this prefix? (bytewise comparison)
+bool String::hasPrefix(const char *rhs) const noexcept
+{
+  int n = strlen(rhs);
+  return (size() >= n && !strncmp(iImp->iData, rhs, n));
+}
+
 //! Equality operator (bytewise comparison).
-bool String::operator==(const String &rhs) const
+bool String::operator==(const String &rhs) const noexcept
 {
   return (size() == rhs.size() &&
-	  ::strncmp(iImp->iData, rhs.iImp->iData, size()) == 0);
+	  !strncmp(iImp->iData, rhs.iImp->iData, size()));
+}
+
+//! Equality operator (bytewise comparison).
+bool String::operator==(const char *rhs) const noexcept
+{
+  int n = strlen(rhs);
+  return (size() == n && !strncmp(iImp->iData, rhs, n));
 }
 
 //! Inequality operator (bytewise comparison).
-bool String::operator<(const String &rhs) const
+bool String::operator<(const String &rhs) const noexcept
 {
   int n = size() < rhs.size() ? size() : rhs.size();
   int cmp = ::strncmp(iImp->iData, rhs.iImp->iData, n);
@@ -254,7 +352,7 @@ bool String::operator<(const String &rhs) const
 }
 
 //! Concatenate this string with \a rhs.
-String String::operator+(const String &rhs) const
+String String::operator+(const String &rhs) const noexcept
 {
   String s(*this);
   s.append(rhs);
@@ -279,7 +377,7 @@ static const uchar firstByteMark[7] = {
 //! Return Unicode value from UTF-8 string.
 /*! The \a index is incremented to the next UTF-8 character.
   This returns 0xfffd if there is any problem in parsing UTF-8. */
-int String::unicode(int &index) const
+int String::unicode(int &index) const noexcept
 {
   int wch = uchar(iImp->iData[index++]);
   if ((wch & 0xc0) == 0x80) {
@@ -446,33 +544,36 @@ void Lex::skipWhitespace()
   Implements sharing semantics using reference counting.
 */
 
-//! Create empty buffer.
-Buffer::Buffer()
-{
-  iImp = new Imp;
-  iImp->iRefCount = 1;
-  iImp->iData = 0;
-  iImp->iSize = 0;
-}
-
 //! Create buffer of specified size.
-Buffer::Buffer(int size)
+Buffer::Buffer(int size) noexcept
 {
   iImp = new Imp;
   iImp->iRefCount = 1;
   iImp->iSize = size;
-  iImp->iData = new char[size];
+  iImp->iData = new (std::nothrow) char[size];
+  if (iImp->iData == nullptr) {
+    fprintf(stderr, "Failure to allocate %d bytes of memory, exiting.\n", size);
+    exit(9);
+  }
 }
 
 //! Copy constructor. Constant time, buffer is shared.
-Buffer::Buffer(const Buffer &rhs)
+Buffer::Buffer(const Buffer &rhs) noexcept
 {
   iImp = rhs.iImp;
-  iImp->iRefCount++;
+  if (iImp)
+    iImp->iRefCount++;
+}
+
+//! Move constructor. Constant time.
+Buffer::Buffer(Buffer &&rhs) noexcept
+{
+  iImp = rhs.iImp;
+  rhs.iImp = nullptr;
 }
 
 //! Create buffer by copying the data.
-Buffer::Buffer(const char *data, int size)
+Buffer::Buffer(const char *data, int size) noexcept
 {
   iImp = new Imp;
   iImp->iRefCount = 1;
@@ -482,26 +583,48 @@ Buffer::Buffer(const char *data, int size)
 }
 
 //! Destructor.
-Buffer::~Buffer()
+Buffer::~Buffer() noexcept
 {
-  iImp->iRefCount--;
-  if (iImp->iRefCount == 0) {
-    delete [] iImp->iData;
-    delete iImp;
-  }
-}
-
-//! Assignment operator (constant-time).
-Buffer &Buffer::operator=(const Buffer &rhs)
-{
-  if (this != &rhs) {
+  if (iImp) {
     iImp->iRefCount--;
     if (iImp->iRefCount == 0) {
       delete [] iImp->iData;
       delete iImp;
     }
+  }
+}
+
+//! Assignment operator (constant-time).
+Buffer &Buffer::operator=(const Buffer &rhs) noexcept
+{
+  if (this != &rhs) {
+    if (iImp) {
+      iImp->iRefCount--;
+      if (iImp->iRefCount == 0) {
+	delete [] iImp->iData;
+	delete iImp;
+      }
+    }
     iImp = rhs.iImp;
-    iImp->iRefCount++;
+    if (iImp)
+      iImp->iRefCount++;
+  }
+  return *this;
+}
+
+//! Assignment move operator (constant-time).
+Buffer &Buffer::operator=(Buffer &&rhs) noexcept
+{
+  if (this != &rhs) {
+    if (iImp) {
+      iImp->iRefCount--;
+      if (iImp->iRefCount == 0) {
+	delete [] iImp->iData;
+	delete iImp;
+      }
+    }
+    iImp = rhs.iImp;
+    rhs.iImp = nullptr;
   }
   return *this;
 }
@@ -759,6 +882,11 @@ BufferSource::BufferSource(const Buffer &buffer)
   iPos = 0;
 }
 
+void BufferSource::setPosition(int pos)
+{
+  iPos = pos;
+}
+
 int BufferSource::getChar()
 {
   if (iPos >= iBuffer.size())
