@@ -655,8 +655,8 @@ bool ImlParser::parseStyle(StyleSheet &sheet)
       String value = att["begin"];
       value += '\0';
       value += att["end"];
-      sheet.add(ETextStyle, Attribute(true, name),
-		Attribute(false, value));
+      Kind k = (att["type"] == "label") ? ELabelStyle : ETextStyle;
+      sheet.add(k, Attribute(true, name), Attribute(false, value));
     } else {
       Kind kind;
       if (tag == "pen")
