@@ -51,6 +51,7 @@ Latex::Latex(const Cascade *sheet, LatexType latexType)
 {
   iCascade = sheet;
   iResources = new PdfResources;
+  iLatexType = latexType;
   iXetex = (latexType == LatexType::Xetex);
 }
 
@@ -186,20 +187,18 @@ int Latex::createLatexSource(Stream &stream, String preamble)
   if (!iXetex) {
     stream << "\\expandafter\\ifx\\csname pdfobjcompresslevel\\endcsname"
 	   << "\\relax\\else\\pdfobjcompresslevel0\\fi\n";
-    if (!ancient) {
+    if (!ancient && iLatexType != LatexType::Luatex)
       stream << "\\ifnum\\the\\pdftexversion<140"
 	     << "\\errmessage{Pdftex is too old. "
 	     << "Set IPEANCIENTPDFTEX environment variable!}\\fi\n";
-    }
   }
   stream << "\\documentclass{article}\n"
-	 << "\\newcommand{\\PageTitle}[1]{#1}\n"
+	 << "\\usepackage{xcolor}\n"
 	 << "\\newdimen\\ipefs\n"
 	 << "\\newcounter{ipePage}\\newcounter{ipeView}\n"
 	 << "\\newcounter{ipePages}\\newcounter{ipeViews}\n"
-	 << "\\newcommand{\\ipesymbol}[4]{\\ipefs 1ex\\pdfliteral"
-	 << "{(#1) (\\the\\ipefs) (#2) (#3) (#4) sym}}\n"
-	 << "\\usepackage{xcolor}\n";
+	 << "\\newcommand{\\PageTitle}[1]{#1}\n"
+	 << "\\newcommand{\\ipesymbol}[4]{$\\bullet$}\n";
   AttributeSeq colors;
   iCascade->allNames(EColor, colors);
   for (AttributeSeq::const_iterator it = colors.begin();
@@ -216,13 +215,13 @@ int Latex::createLatexSource(Stream &stream, String preamble)
 	     << value.iBlue << "}\n";
   }
   if (iXetex) {
-    stream << "\\def\\ipesetcolor{\\special{pdf:bc [0 0 0 0]}}\n"
+    stream << "\\def\\ipesetcolor#1#2#3{\\special{pdf:bc [#1 #2 #3]}}\n"
 	   << "\\def\\iperesetcolor{\\special{pdf:ec}}\n";
   } else if (!ancient) {
-    stream << "\\def\\ipesetcolor{\\pdfcolorstack0 push{0 0 0 0 k 0 0 0 0 K}}\n"
+    stream << "\\def\\ipesetcolor#1#2#3{\\pdfcolorstack0 push{#1 #2 #3 rg #1 #2 #3 RG}}\n"
 	   << "\\def\\iperesetcolor{\\pdfcolorstack0 pop}\n";
   } else {
-    stream << "\\def\\ipesetcolor{\\color[cmyk]{0,0,0,0}}\n"
+    stream << "\\def\\ipesetcolor#1#2#3{\\color[rgb]{#1,#2,#3}}\n"
 	   << "\\def\\iperesetcolor{}\n";
   }
   stream << iCascade->findPreamble() << "\n"
@@ -260,15 +259,20 @@ int Latex::createLatexSource(Stream &stream, String preamble)
 	     << "{" << fs.mult(6, 5) << "bp}\\selectfont\n";
     } else
       stream << fsAttr.string() << "\n";
-    stream << "\\ipesetcolor\n";
+    Color col = iCascade->find(EColor, text->stroke()).color();
+    stream << "\\ipesetcolor{" << col.iRed.toDouble()
+	   << "}{" << col.iGreen.toDouble()
+	   << "}{" << col.iBlue.toDouble()
+	   << "}%\n";
 
-    Attribute absStyle = iCascade->find(ETextStyle, text->style());
+    Attribute absStyle =
+      iCascade->find(text->isMinipage() ? ETextStyle : ELabelStyle,
+		     text->style());
     String style = absStyle.string();
     int sp = 0;
     while (sp < style.size() && style[sp] != '\0')
       ++sp;
-    if (text->isMinipage())
-      stream << style.substr(0, sp);
+    stream << style.substr(0, sp);
 
     String txt = text->text();
     stream << txt;
@@ -279,7 +283,7 @@ int Latex::createLatexSource(Stream &stream, String preamble)
       stream << style.substr(sp + 1);
       stream << "\\end{minipage}";
     } else
-      stream << "%\n";
+      stream << style.substr(sp + 1) << "%\n";
 
     stream << "\\iperesetcolor}\n"
 	   << "\\count0=\\dp0\\divide\\count0 by \\bigpoint\n";
@@ -314,16 +318,8 @@ int Latex::createLatexSource(Stream &stream, String preamble)
   return count;
 }
 
-bool Latex::getXForm(const PdfObj *xform, const PdfDict *ipeInfo)
+bool Latex::getXForm(String key, const PdfDict *ipeInfo)
 {
-  const PdfDict *dict = xform->dict();
-  if (!dict || dict->stream().size() == 0)
-    return false;
-  Text::XForm *xf = new Text::XForm;
-  iXForms.push_back(xf);
-  xf->iStream = dict->inflate();
-  // ipeDebug("stream is <%s>",
-  // String(xf->iStream.data(), xf->iStream.size()).z());
   /*
      /Type /XObject
      /Subtype /Form
@@ -335,6 +331,34 @@ bool Latex::getXForm(const PdfObj *xform, const PdfDict *ipeInfo)
      /Matrix [1 0 0 1 0 0]
      /Resources 11 0 R
   */
+  Text::XForm *xf = new Text::XForm;
+  iXForms.push_back(xf);
+  const PdfObj *xform = iXetex ? ipeInfo->get("IpeXForm", nullptr) :
+    iResources->findResource("XObject", key);
+  int xformNum = -1;
+  if (xform && xform->ref()) {
+    xformNum = xform->ref()->value();
+    xform = iResources->object(xformNum);
+  }
+  if (!xform || !xform->dict())
+    return false;
+  const PdfDict *xformd = xform->dict();
+  if (iXetex) {
+    // determine key
+    const PdfDict *d = iResources->resourcesOfKind("XObject");
+    for (int i = 0; i < d->count(); ++i) {
+      const PdfObj *obj = d->value(i);
+      if (obj->ref() && obj->ref()->value() == xformNum) {
+	xf->iName = d->key(i);
+	break;
+      }
+    }
+    if (xf->iName.empty())
+      return false;
+  } else {
+    xf->iName = key;
+    ipeInfo = xformd;
+  }
   double val;
   // Get  id
   if (!ipeInfo->getNumber("IpeId", val, &iPdf))
@@ -349,19 +373,19 @@ bool Latex::getXForm(const PdfObj *xform, const PdfDict *ipeInfo)
 
   // Get BBox
   std::vector<double> a;
-  if (!dict->getNumberArray("BBox", &iPdf, a) || a.size() != 4)
+  if (!xformd->getNumberArray("BBox", &iPdf, a) || a.size() != 4)
     return false;
   xf->iBBox.addPoint(Vector(a[0], a[1]));
   xf->iBBox.addPoint(Vector(a[2], a[3]));
 
-  if (!dict->getNumberArray("Matrix", &iPdf, a) || a.size() != 6)
+  if (!xformd->getNumberArray("Matrix", &iPdf, a) || a.size() != 6)
     return false;
   if (a[0] != 1.0 || a[1] != 0.0 || a[2] != 0.0 || a[3] != 1.0) {
     ipeDebug("PDF XObject has a non-trivial transformation");
     return false;
   }
-  // xf->iOrigin = Vector(-a[4], -a[5]);
-  return iResources->collect(dict, &iPdf);
+  xf->iTranslation = Vector(-a[4], -a[5]) - xf->iBBox.bottomLeft();
+  return true;
 }
 
 //! Read the PDF file created by Pdflatex.
@@ -376,10 +400,14 @@ bool Latex::readPdf(DataSource &source)
   }
 
   const PdfDict *page1 = iPdf.page();
+
   const PdfObj *res = page1->get("Resources", &iPdf);
   if (!res || !res->dict())
     return false;
 
+  if (!iResources->collect(res->dict(), &iPdf))
+    return false;
+
   if (iXetex) {
     const PdfObj *obj = res->dict()->get("Ipe", &iPdf);
     if (!obj || !obj->array()) {
@@ -390,8 +418,7 @@ bool Latex::readPdf(DataSource &source)
       const PdfObj *info = obj->array()->obj(i, &iPdf);
       if (!info || !info->dict())
 	return false;
-      const PdfObj *xform = info->dict()->get("IpeXForm", &iPdf);
-      if (!getXForm(xform, info->dict()))
+      if (!getXForm(String(), info->dict()))
 	return false;
     }
   } else {
@@ -400,11 +427,9 @@ bool Latex::readPdf(DataSource &source)
       warn("Page 1 has no XForms.");
       return false;
     }
-    // collect list of XObject's and their fonts
     for (int i = 0; i < obj->dict()->count(); i++) {
       String key = obj->dict()->key(i);
-      const PdfObj *xform = obj->dict()->get(key, &iPdf);
-      if (!getXForm(xform, xform->dict()))
+      if (!getXForm(key, nullptr))
 	return false;
     }
   }
