@@ -54,7 +54,10 @@ static const char *styleStandard[] = {
   "<pagenumberstyle pos=\"10 10\" size=\"normal\" color=\"black\"/>",
   "<pathstyle cap=\"0\" join=\"1\" fillrule=\"eofill\"/>",
   "<textstyle name=\"normal\" begin=\"\" end=\"\"/>",
+  "<textstyle name=\"normal\" type=\"label\" begin=\"\" end=\"\"/>",
+  "<textstyle name=\"math\" type=\"label\" begin=\"$\" end=\"$\"/>",
   "<opacity name=\"opaque\" value=\"1\"/>",
+  "<textpad left=\"0\" right=\"0\" top=\"1\" bottom=\"1\"/>",
   "</ipestyle>",
   0 };
 
